@@ -1,4 +1,4 @@
-#   Copyright 2024 The PyMC Developers
+#   Copyright 2024 - present The PyMC Developers
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -328,8 +328,7 @@ class Prod(Combination):
         check = Counter([isinstance(factor, Covariance) for factor in self._factor_list])
         if check.get(True, 0) >= 2:
             raise NotImplementedError(
-                "The power spectral density of products of covariance "
-                "functions is not implemented."
+                "The power spectral density of products of covariance functions is not implemented."
             )
         return reduce(mul, self._merge_factors_psd(omega))
 
