@@ -1,4 +1,4 @@
-#   Copyright 2024 The PyMC Developers
+#   Copyright 2024 - present The PyMC Developers
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -22,7 +22,6 @@ import pytensor
 import pytensor.tensor as pt
 import scipy.sparse as sps
 
-from pytensor import scalar
 from pytensor.compile import Function, Mode, get_mode
 from pytensor.compile.builders import OpFromGraph
 from pytensor.gradient import grad
@@ -415,31 +414,6 @@ def hessian_diag(f, vars=None, negate_output=True):
         return empty_gradient
 
 
-class IdentityOp(scalar.UnaryScalarOp):
-    @staticmethod
-    def st_impl(x):
-        return x
-
-    def impl(self, x):
-        return x
-
-    def grad(self, inp, grads):
-        return grads
-
-    def c_code(self, node, name, inp, out, sub):
-        return f"{out[0]} = {inp[0]};"
-
-    def __eq__(self, other):
-        return isinstance(self, type(other))
-
-    def __hash__(self):
-        return hash(type(self))
-
-
-scalar_identity = IdentityOp(scalar.upgrade_to_float, name="scalar_identity")
-identity = Elemwise(scalar_identity, name="identity")
-
-
 def make_shared_replacements(point, vars, model):
     """
     Make shared replacements for all *other* variables than the ones passed.
@@ -881,35 +855,44 @@ def collect_default_updates(
 
         # Root case, RNG is not used elsewhere
         if not rng_clients:
-            return rng
+            return None
 
         if len(rng_clients) > 1:
             # Multiple clients are techincally fine if they are used in identical operations
             # We check if the default_update of each client would be the same
-            update, *other_updates = (
+            all_updates = [
                 find_default_update(
                     # Pass version of clients that includes only one the RNG clients at a time
                     clients | {rng: [rng_client]},
                     rng,
                 )
                 for rng_client in rng_clients
-            )
-            if all(equal_computations([update], [other_update]) for other_update in other_updates):
-                return update
+            ]
+            updates = [update for update in all_updates if update is not None]
+            if not updates:
+                return None
+            if len(updates) == 1:
+                return updates[0]
+            else:
+                update, *other_updates = updates
+                if all(
+                    equal_computations([update], [other_update]) for other_update in other_updates
+                ):
+                    return update
 
-            warnings.warn(
-                f"RNG Variable {rng} has multiple distinct clients {rng_clients}, "
-                f"likely due to an inconsistent random graph. "
-                f"No default update will be returned.",
-                UserWarning,
-            )
-            return None
+                warnings.warn(
+                    f"RNG Variable {rng} has multiple distinct clients {rng_clients}, "
+                    f"likely due to an inconsistent random graph. "
+                    f"No default update will be returned.",
+                    UserWarning,
+                )
+                return None
 
         [client, _] = rng_clients[0]
 
         # RNG is an output of the function, this is not a problem
         if isinstance(client.op, Output):
-            return rng
+            return None
 
         # RNG is used by another operator, which should output an update for the RNG
         if isinstance(client.op, RandomVariable):
@@ -938,18 +921,26 @@ def collect_default_updates(
                 )
         elif isinstance(client.op, OpFromGraph):
             try:
-                next_rng = collect_default_updates_inner_fgraph(client)[rng]
-            except (ValueError, KeyError):
+                next_rng = collect_default_updates_inner_fgraph(client).get(rng)
+                if next_rng is None:
+                    # OFG either does not make use of this RNG or inconsistent use that will have emitted a warning
+                    return None
+            except ValueError as exc:
                 raise ValueError(
                     f"No update found for at least one RNG used in OpFromGraph Op {client.op}.\n"
                     "You can use `pytensorf.collect_default_updates` and include those updates as outputs."
-                )
+                ) from exc
         else:
             # We don't know how this RNG should be updated. The user should provide an update manually
             return None
 
         # Recurse until we find final update for RNG
-        return find_default_update(clients, next_rng)
+        nested_next_rng = find_default_update(clients, next_rng)
+        if nested_next_rng is None:
+            # There were no more uses of this next_rng
+            return next_rng
+        else:
+            return nested_next_rng
 
     if inputs is None:
         inputs = []
