@@ -1,4 +1,4 @@
-#   Copyright 2024 The PyMC Developers
+#   Copyright 2024 - present The PyMC Developers
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -318,8 +318,8 @@ class TestData:
         N_cols = 7
         data = np.random.uniform(size=(N_rows, N_cols))
         coords = {
-            "rows": [f"R{r+1}" for r in range(N_rows)],
-            "columns": [f"C{c+1}" for c in range(N_cols)],
+            "rows": [f"R{r + 1}" for r in range(N_rows)],
+            "columns": [f"C{c + 1}" for c in range(N_cols)],
         }
         # pass coordinates explicitly, use numpy array in Data container
         with pm.Model(coords=coords) as pmodel:
@@ -375,7 +375,7 @@ class TestData:
         pd = pytest.importorskip("pandas")
         ser_sales = pd.Series(
             data=np.random.randint(low=0, high=30, size=22),
-            index=pd.date_range(start="2020-05-01", periods=22, freq="24H", name="date"),
+            index=pd.date_range(start="2020-05-01", periods=22, freq="24h", name="date"),
             name="sales",
         )
         with pm.Model() as pmodel:
@@ -391,7 +391,7 @@ class TestData:
         N_cols = 7
         df_data = pd.DataFrame()
         for c in range(N_cols):
-            df_data[f"Column {c+1}"] = np.random.normal(size=(N_rows,))
+            df_data[f"Column {c + 1}"] = np.random.normal(size=(N_rows,))
         df_data.index.name = "rows"
         df_data.columns.name = "columns"
 
